#include <QtGui>

#include "devicedialog.h"


DeviceDialog::DeviceDialog(DiagramDevice *device, QWidget *parent)
    : QDialog(parent)
{
	generaltab = new GeneralTab(device);
	modeltab = new ModelTab(device);
    tabWidget = new QTabWidget;
    tabWidget->addTab(generaltab, tr("General"));
    tabWidget->addTab(modeltab, tr("Model"));


    buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
//! [2] //! [3]

//! [4]
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget);
    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
//! [4]

//! [5]
    setWindowTitle(tr("Properties"));
	resize(400,400);
}

void DeviceDialog::accept()
{
	generaltab->saveInput();
	QDialog::accept();
}

GeneralTab::GeneralTab(DiagramDevice *device, QWidget *parent)
    : QWidget(parent)
{
	dev = device;

    QLabel *deviceNameLabel = new QLabel(tr("Device Name:"));
    deviceNameEdit = new QLineEdit(device->deviceName());

    QLabel *deviceValueLabel = new QLabel(tr("Device Value:"));
    deviceValueEdit = new QLineEdit;
	if(device->deviceType() > DiagramDevice::Isrc){
		deviceValueEdit->setText(tr("N/A"));
		deviceValueEdit->setReadOnly(true);
	}else deviceValueEdit->setText(device->deviceValue());

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(deviceNameLabel);
    mainLayout->addWidget(deviceNameEdit);
    mainLayout->addWidget(deviceValueLabel);
    mainLayout->addWidget(deviceValueEdit);

    mainLayout->addStretch(1);
    setLayout(mainLayout);
}

void GeneralTab::saveInput()
{
	if(deviceNameEdit->isModified()){
		if(!deviceNameEdit->text().isEmpty()){
			dev->setName(deviceNameEdit->text(),this);
		}
	}

	if(deviceValueEdit->isModified()){
		if(!deviceValueEdit->text().isEmpty()){
			dev->setValue(deviceValueEdit->text());
		}
	}

}

ModelTab::ModelTab(DiagramDevice *device, QWidget *parent)
    : QWidget(parent)
{
    dev = device;

	QLabel *widthLabel = new QLabel(tr("Width:"));
    widthEdit = new QLineEdit();

    QLabel *lengthLabel = new QLabel(tr("Length:"));
    lengthEdit = new QLineEdit;

	QLabel *modelNameLabel = new QLabel(tr("Model Name:"));
    modelNameEdit = new QLineEdit;

	if(device->deviceType() <= DiagramDevice::Isrc){
		widthEdit->setText(tr("N/A"));
		lengthEdit->setText(tr("N/A"));
		modelNameEdit->setText(tr("N/A"));
		widthEdit->setReadOnly(true);
		lengthEdit->setReadOnly(true);
		modelNameEdit->setReadOnly(true);
	}

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(widthLabel);
    mainLayout->addWidget(widthEdit);
    mainLayout->addWidget(lengthLabel);
    mainLayout->addWidget(lengthEdit);
	mainLayout->addWidget(modelNameLabel);
    mainLayout->addWidget(modelNameEdit);

    mainLayout->addStretch(1);
    setLayout(mainLayout);
}

void ModelTab::saveInput()
{}