#include <QtGui>

#include "diagramdevice.h"
#include "wire.h"
#include "devicedialog.h"

//! [0]
DiagramDevice::DiagramDevice(DeviceType deviceType, QMenu *contextMenu,
             QGraphicsItem *parent, QGraphicsScene *scene)
    : QGraphicsPixmapItem(parent, scene)
{
    myDeviceType = deviceType;
    myContextMenu = contextMenu;

    QPainterPath path;
	switch (myDeviceType) {
        case Resistor:
			setPixmap(QPixmap(":/images/res.png","PNG"));
			portIndex = QVector<int>(2,-1);
			wires.resize(2);
			name = "R";
            break;
        case Capacitor:
			setPixmap(QPixmap(":/images/cap.png","PNG"));
			portIndex = QVector<int>(2,-1);
			wires.resize(2);
			name = "C";
            break;
        case Inductor:
			setPixmap(QPixmap(":/images/ind.png","PNG"));
			portIndex = QVector<int>(2,-1);
			wires.resize(2);
			name = "L";
            break;
		case Vsrc:
			setPixmap(QPixmap(":/images/vsrc.png","PNG"));
			portIndex = QVector<int>(2,-1);
			wires.resize(2);
			name = "V";
			break;
		case Isrc:
			setPixmap(QPixmap(":/images/isrc.png","PNG"));
			portIndex = QVector<int>(2,-1);
			wires.resize(2);
			name = "I";
			break;
		case NMOS:
			setPixmap(QPixmap(":/images/nmos.png","PNG"));
			portIndex = QVector<int>(3,-1);
			wires.resize(3);
			name = "M";
			break;
		case PMOS:
			setPixmap(QPixmap(":/images/pmos.png","PNG"));
			portIndex = QVector<int>(3,-1);
			wires.resize(3);
			name = "M";
			break;
		case PNP:
			setPixmap(QPixmap(":/images/pnp.png","PNG"));
			portIndex = QVector<int>(3,-1);
			wires.resize(3);
			name = "Q";
			break;
		case NPN:
			setPixmap(QPixmap(":/images/npn.png","PNG"));
			portIndex = QVector<int>(3,-1);
			wires.resize(3);
			name = "Q";
			break;
		case GND:
			setPixmap(QPixmap(":/images/gnd.png","PNG"));
			portIndex = QVector<int>(1,-1);
			wires.resize(1);
			break;
		default:
            break;
    }
    setFlag(QGraphicsItem::ItemIsMovable, true);
    setFlag(QGraphicsItem::ItemIsSelectable, true);
    setFlag(QGraphicsItem::ItemSendsGeometryChanges, true);
	setShapeMode(QGraphicsPixmapItem::BoundingRectShape);
}
//! [0]

//! [1]
void DiagramDevice::removeWire(Wire *wire)
{
    int index = -1;
	int i = 0;
	while(index < 0 && i < wires.size())
		index = wires[i++].indexOf(wire);

    if (index != -1)
        wires[i-1].removeAt(index);
}
//! [1]

//! [2]
void DiagramDevice::removeWires()
{
	int i = 0;
	while(i<wires.size()){
		foreach (Wire *wire, wires[i]) {
			wire->startDevice()->removeWire(wire);
			wire->endDevice()->removeWire(wire);
			scene()->removeItem(wire);
			delete wire;
		}
		i++;
	}
}
//! [2]

//! [3]
void DiagramDevice::addWire(Wire *wire, int portIndex)
{
    wires[portIndex].append(wire);
}
//! [3]

//! [4]
QPixmap DiagramDevice::image() const
{
	return pixmap();
}
//! [4]

//! [5]
void DiagramDevice::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
{
    scene()->clearSelection();
    setSelected(true);
    myContextMenu->exec(event->screenPos());
}
//! [5]

//! [6]
QVariant DiagramDevice::itemChange(GraphicsItemChange change,
                     const QVariant &value)
{
    if (change == QGraphicsItem::ItemPositionChange) {
		int i=0;
		while(i<wires.size()){
			foreach (Wire *wire, wires[i]) {
		//		wire->updatePosition();
			}
			i++;
		}
    }

    return value;
}
//! [6]

void DiagramDevice::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
{
	DeviceDialog *devicedialog = new DeviceDialog(this);
	devicedialog->exec();
	delete devicedialog;
}

QPointF DiagramDevice::getClosestPort(QPointF pos)
{
	switch(myDeviceType){
		case Resistor:
		case Capacitor:
		case Inductor:
		case Vsrc:
		case Isrc:
			if(pos.y()<=pixmap().height()/2)
				return QPointF(pixmap().width()/2,0);
			else return QPointF(pixmap().width()/2,pixmap().height());
		case DiagramDevice::NPN:
		case DiagramDevice::PNP:
		case DiagramDevice::NMOS:
		case DiagramDevice::PMOS:
			if(pos.x()<=pixmap().width()/2)
				return QPointF(0,pixmap().height()/2);;
			if(pos.y()<=pixmap().height()/2)
				return QPointF(pixmap().width(),pixmap().height()/4);
			return QPointF(pixmap().width(),pixmap().height()*3/4);
		case DiagramDevice::GND:
			return QPointF(pixmap().width()/2,0);
		default:
			return QPointF(pixmap().width()/2,0);
	}
}
void DiagramDevice::setName(QString str, QWidget *parent)
{
	name = str;

	switch(myDeviceType){
		case Resistor:
			if(name[0] == QChar('r') || name[0] == QChar('R'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Resistor name should start with the charactor \"R/r\". "));
			name = "R";
			break;
		case Capacitor:
			if(name[0] == QChar('c') || name[0] == QChar('C'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Capacitor name should start with the charactor \"C/c\". "));
			name = "C";
			break;
		case Inductor:
			if(name[0] == QChar('l') || name[0] == QChar('L'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Inductor name should start with the charactor \"L/l\". "));
			name = "L";
			break;
		case Vsrc:
			if(name[0] == QChar('v') || name[0] == QChar('V'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Voltage source name should start with the charactor \"V/v\". "));
			name = "V";
			break;
		case Isrc:
			if(name[0] == QChar('i') || name[0] == QChar('I'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Current source name should start with the charactor \"I/i\". "));
			name = "I";
			break;
		case NMOS:
		case PMOS:
			if(name[0] == QChar('m') || name[0] == QChar('M'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("MOSFET transistor name should start with the charactor \"M/m\". "));
			name = "M";
			break;
		case PNP:
		case NPN:
			if(name[0] == QChar('q') || name[0] == QChar('Q'))
				return;
			QMessageBox::information(parent, QObject::tr("Warning"), QObject::tr("Bipolr transistor name should start with the charactor \"Q/q\". "));
			name = "Q";
			break;
	}
}

void DiagramDevice::setValue(QString str)
{
	value = str;
}
