/*
 * DiagramDevice defines different devices in schematics.
 * myDeviceType --- Different devices are distinguished by this enum variable. More devices can be easily added by defining new enum elements.
 * wires --- wires that are connected to current device. They're sorted into different catogeries based on which port they're connected to.
*/
#ifndef DIAGRAMDEVICE_H
#define DIAGRAMDEVICE_H

#include <QGraphicsPixmapItem>
#include <QList>

QT_BEGIN_NAMESPACE
class QPixmap;
class QGraphicsItem;
class QGraphicsScene;
class QTextEdit;
class QGraphicsSceneMouseEvent;
class QMenu;
class QGraphicsSceneContextMenuEvent;
class QPainter;
class QStyleOptionGraphicsItem;
class QWidget;
class QPolygonF;
QT_END_NAMESPACE

class Wire;

//! [0]
class DiagramDevice : public QGraphicsPixmapItem
{
public:
    enum { Type = UserType + 15 };
    enum DeviceType { Resistor, Capacitor, Inductor, Vsrc, Isrc, NMOS, PMOS, PNP, NPN, GND };

    DiagramDevice(DeviceType deviceType, QMenu *contextMenu,
        QGraphicsItem *parent = 0, QGraphicsScene *scene = 0);

    void removeWire(Wire *wire);
    void removeWires();
    DeviceType deviceType() const
        { return myDeviceType; }
    void addWire(Wire *wire, int portIndex);
	QList<Wire *> getWires(int portIndex){ return wires[portIndex]; }
    QPixmap image() const;
    int type() const
        { return Type;}

	// Set circuit node number of a device port
	// index --- id of the device port
	// val --- node number in the circuit
	void setPortNum(int index, int val)
		{ if(index<portIndex.size()) portIndex[index] = val; }
	
	// Return circuit node number of a device port
	int getPortNum(int index)
		{ if(index<portIndex.size()) return portIndex[index]; else return -1; }

	//input scene coordinate and return the coordinate of device port that is the closest to the input coordinate
	QPointF getClosestPort(QPointF pos);

	void setValue(QString str);
	void setName(QString str, QWidget *parent);
	QString deviceName(){return name;}
	QString deviceValue(){return value;}

protected:
    void contextMenuEvent(QGraphicsSceneContextMenuEvent *event);
   	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event); //this function is ignored before mouse double click event for diagram scene has higher priority
	QVariant itemChange(GraphicsItemChange change, const QVariant &value);


private:
    DeviceType myDeviceType;
    QMenu *myContextMenu;
    QVector<QList<Wire *> > wires;
	QVector<int> portIndex;
	QString value;
	QString name;

};
//! [0]

#endif
