/*
 * DiagramScene manipulates devices and wires in schematic.
 * It contains data structures of devices and wires implicitly. 
*/

#ifndef DIAGRAMSCENE_H
#define DIAGRAMSCENE_H

#include <QGraphicsScene>
#include "diagramdevice.h"
#include "diagramtextitem.h"

QT_BEGIN_NAMESPACE
class QGraphicsSceneMouseEvent;
class QMenu;
class QPointF;
class QGraphicsLineItem;
class QFont;
class QGraphicsTextItem;
class QColor;
QT_END_NAMESPACE


class DiagramScene : public QGraphicsScene
{
    Q_OBJECT

public:
    enum Mode { InsertDev, InsertLine, InsertText, MoveItem };

    DiagramScene(QMenu *itemMenu, QObject *parent = 0);
    QFont font() const
        { return myFont; }
    QColor textColor() const
        { return myTextColor; }
    QColor itemColor() const
        { return myItemColor; }
    QColor lineColor() const
        { return myLineColor; }
    void setLineColor(const QColor &color);
    void setTextColor(const QColor &color);
    void setItemColor(const QColor &color);
    void setFont(const QFont &font);

public slots:
    void setMode(Mode mode);
    void setItemType(DiagramDevice::DeviceType type);
    void editorLostFocus(DiagramTextItem *item);

signals:
    void itemInserted(DiagramDevice *item);
    void textInserted(QGraphicsTextItem *item);
    void itemSelected(QGraphicsItem *item);

protected:
    void mousePressEvent(QGraphicsSceneMouseEvent *mouseEvent);
    void mouseMoveEvent(QGraphicsSceneMouseEvent *mouseEvent);
 //   void mouseReleaseEvent(QGraphicsSceneMouseEvent *mouseEvent);
	void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *mouseEvent);

private:
    bool isItemChange(int type);

    DiagramDevice::DeviceType myItemType;
    QMenu *myItemMenu;
    Mode myMode;
    QPointF startPoint;
	
	//Movable wire that follows cursor when drawing wires in schematic
	QGraphicsPathItem *dynmPath;
	
	//Static wire fixed in schematic temporarily by clicking cursor on the canvas
	QGraphicsPathItem *statPath;
    
	QFont myFont;
    DiagramTextItem *textItem;
    QColor myTextColor;
    QColor myItemColor;
    QColor myLineColor;
};

#endif
