#include <QtGui>

#include "wire.h"
#include <math.h>

const qreal Pi = 3.14;

//! [0]
Wire::Wire(DiagramDevice *startDevice, DiagramDevice *endDevice,
         QGraphicsItem *parent, QGraphicsScene *scene)
    : QGraphicsPathItem(parent, scene)
{
    myStartDevice = startDevice;
    myEndDevice = endDevice;
    setFlag(QGraphicsItem::ItemIsSelectable, true);
    myColor = Qt::black;
    setPen(QPen(myColor, 2, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
	startPort = 0;
	endPort = 0;
	marked = false;
}
//! [0]
/*
//! [1]
QRectF Wire::boundingRect() const
{
    qreal extra = (pen().width() + 20) / 2.0;

    return QRectF(line().p1(), QSizeF(line().p2().x() - line().p1().x(),
                                      line().p2().y() - line().p1().y()))
        .normalized()
        .adjusted(-extra, -extra, extra, extra);
}
//! [1]


//! [3]
void Wire::updatePosition()
{
    QLineF line(mapFromItem(myStartDevice, 100, 100), mapFromItem(myEndDevice, 100, 100));
	//QLineF line(QPointF(10,10), QPointF(100,100));
    setLine(line);
}*/
//! [3]

//! [4]
void Wire::paint(QPainter *painter, const QStyleOptionGraphicsItem *,
          QWidget *)
{
    if (myStartDevice->collidesWithItem(myEndDevice))
        return;

    QPen myPen = pen();
    myPen.setColor(myColor);
	if (isSelected())
		myPen.setStyle(Qt::DashLine);
	else
		myPen.setStyle(Qt::SolidLine);

    painter->setPen(myPen);
	painter->setBrush(Qt::NoBrush);

	QPointF p1 = myStartDevice->mapToItem(this,createPortPos(myStartDevice,startPort));
	QPointF p2 = myEndDevice->mapToItem(this,createPortPos(myEndDevice,endPort));

	if(contains(p1) == false || contains(p2) == false){//if devices are moved, connection between them should be updated
		QPainterPath newPath;
		QPointF p1,p2;
		p1 = myStartDevice->mapToScene(createPortPos(myStartDevice,startPort));
		p2 = myEndDevice->mapToScene(createPortPos(myEndDevice,endPort));
		newPath.moveTo(p1);
		newPath.lineTo(p2.x(),p1.y());
		if(newPath.intersects(myStartDevice->shape())){
			newPath.moveTo(p1);
			newPath.lineTo(p1.x(),p2.y());
		}
		newPath.lineTo(p2);
		setPath(newPath);
	}

	painter->drawPath(path());
}
//! [7]

int Wire::PortIndex(DiagramDevice *dev, QPointF pos)
{
	switch(dev->deviceType()){
		case DiagramDevice::Resistor:
		case DiagramDevice::Capacitor:
		case DiagramDevice::Inductor:
		case DiagramDevice::Vsrc:
		case DiagramDevice::Isrc:
			if(pos.y()<=dev->pixmap().height()/2)
				return 0;
			else return 1;
		case DiagramDevice::NPN:
		case DiagramDevice::PNP:
			if(pos.x()<=dev->pixmap().width()/2)
				return 0;
			else if(pos.y()<=dev->pixmap().height()/2)
				return 1;
			else return 2;
		case DiagramDevice::NMOS:
		case DiagramDevice::PMOS:
			if(pos.x()<=dev->pixmap().width()/2)
				return 0;
			else if(pos.y()<=dev->pixmap().height()/2)
				return 1;
			else return 2;
		case DiagramDevice::GND:
			return 0;
		default:
			return 0;
	}
}

QPointF Wire::createPortPos(DiagramDevice *dev, int PortIndex)
{
	switch(dev->deviceType()){
		case DiagramDevice::Resistor:
		case DiagramDevice::Capacitor:
		case DiagramDevice::Inductor:
		case DiagramDevice::Vsrc:
		case DiagramDevice::Isrc:
			if( PortIndex == 0 )
				return QPointF(dev->pixmap().width()/2,0);
			else return QPointF(dev->pixmap().width()/2,dev->pixmap().height());
		case DiagramDevice::NPN:
		case DiagramDevice::PNP:
			if( PortIndex == 0 )
				return QPointF(0,dev->pixmap().height()/2);
			else if( PortIndex == 1 )
				return QPointF(dev->pixmap().width(),0);
			else return QPointF(dev->pixmap().width(),dev->pixmap().height());
		case DiagramDevice::NMOS:
		case DiagramDevice::PMOS:
			if( PortIndex == 0 )
				return QPointF(0,dev->pixmap().height()/2);
			else if( PortIndex == 1 )
				return QPointF(dev->pixmap().width(),dev->pixmap().height()/4);
			else return QPointF(dev->pixmap().width(),dev->pixmap().height()*3/4);
		case DiagramDevice::GND:
			return QPointF(dev->pixmap().width()/2,0);
		default:
			return QPointF(0,0);
	}
}