/*
 * Wire defines interconnection in schematics.
*/

#ifndef WIRE_H
#define WIRE_H

#include <QGraphicsPathItem>

#include "diagramdevice.h"

QT_BEGIN_NAMESPACE
class QGraphicsPolygonItem;
class QGraphicsPixmapItem;
class QGraphicsPathItem;
class QGraphicsScene;
class QRectF;
class QGraphicsSceneMouseEvent;
class QPainterPath;
QT_END_NAMESPACE

//! [0]
class Wire : public QGraphicsPathItem
{
public:
    enum { Type = UserType + 4 };

    Wire(DiagramDevice *startItem, DiagramDevice *endItem,
      QGraphicsItem *parent = 0, QGraphicsScene *scene = 0);

    int type() const
        { return Type; }
  //  QRectF boundingRect() const;
    void setColor(const QColor &color)
        { myColor = color; }
    DiagramDevice *startDevice() const
        { return myStartDevice; }
    DiagramDevice *endDevice() const
        { return myEndDevice; }

	void setStartPortIndex(QPointF pos)
		{ startPort = PortIndex(myStartDevice,pos); }
	void setEndPortIndex(QPointF pos)
		{ endPort = PortIndex(myEndDevice,pos); }

	// Return the index of port on start device that's connected to current wire
	int startPortIndex(){return startPort;}
	// Return the index of port on end device that's connected to current wire
	int endPortIndex(){return endPort;}

	void mark(){ marked = true; }
	void unmark(){ marked = false; }
	bool isMarked(){return marked;}

protected:
	//this function draws the shape of wire. It's invoked automatically.
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
               QWidget *widget = 0);

private:
    DiagramDevice *myStartDevice;
    DiagramDevice *myEndDevice;
    QColor myColor;
    int startPort;
	int endPort;
	bool marked;

	int PortIndex(DiagramDevice *dev, QPointF pos);
	QPointF createPortPos(DiagramDevice *dev, int PortIndex);
};
//! [0]

#endif
